import { TypedEmitter } from "tiny-typed-emitter";
const InterfaceType = "gpio";
export var GPIOState;
(function (GPIOState) {
    GPIOState[GPIOState["Unknown"] = 0] = "Unknown";
    GPIOState[GPIOState["Error"] = 1] = "Error";
    GPIOState[GPIOState["StrongLow"] = 2] = "StrongLow";
    GPIOState[GPIOState["StrongHigh"] = 3] = "StrongHigh";
    GPIOState[GPIOState["HighZ"] = 4] = "HighZ";
    GPIOState[GPIOState["WeakLow"] = 5] = "WeakLow";
    GPIOState[GPIOState["WeakHigh"] = 6] = "WeakHigh";
})(GPIOState || (GPIOState = {}));
export class GPIOInterface extends TypedEmitter {
    constructor(configuration) {
        super();
        this.interfaceType = InterfaceType;
        this.signalState = GPIOState.Unknown;
        this.driverStates = new Map();
        this.lastDriverState = GPIOState.Unknown;
        this.configuration = configuration;
    }
    // changes the driver of this device
    changeDriver(state) {
        if (state != this.lastDriverState) {
            this.lastDriverState = state;
            const data = {
                driver: this.configuration.driver,
                state: state,
            };
            this.emit("upstreamData", data);
            this.downstreamData(data); // use the same mechanismn as any other driver data from external devices
        }
    }
    // handles downstream data
    downstreamData(data) {
        this.driverStates.set(data.driver, data.state);
        this.evaluateSignalState();
    }
    evaluateSignalState() {
        const states = new Set();
        for (const [_driver, state] of this.driverStates) {
            states.add(state);
        }
        let newState = GPIOState.Unknown;
        if (states.has(GPIOState.Error)) {
            newState = GPIOState.Error;
        }
        else if (states.has(GPIOState.StrongHigh) && states.has(GPIOState.StrongLow)) {
            newState = GPIOState.Error;
        }
        else if (states.has(GPIOState.Unknown)) {
            newState = GPIOState.Unknown;
        }
        else if (states.has(GPIOState.StrongHigh)) {
            newState = GPIOState.StrongHigh;
        }
        else if (states.has(GPIOState.StrongLow)) {
            newState = GPIOState.StrongLow;
        }
        else if (states.has(GPIOState.WeakHigh) && states.has(GPIOState.WeakLow)) {
            newState = GPIOState.Unknown;
        }
        else if (states.has(GPIOState.WeakHigh)) {
            newState = GPIOState.WeakHigh;
        }
        else if (states.has(GPIOState.WeakLow)) {
            newState = GPIOState.WeakLow;
        }
        if (newState !== this.signalState) {
            const event = { oldState: this.signalState, state: newState };
            this.signalState = newState;
            this.emit("signalChange", event);
        }
    }
}
export class ConstructableGPIOInterface {
    constructor(gpios) {
        this.interfaceType = InterfaceType;
        this.gpios = gpios;
    }
    getDescription() {
        return {
            availableSignals: { gpio: this.gpios },
        };
    }
    create(configuration) {
        return new GPIOInterface(configuration);
    }
}
//# sourceMappingURL=connectionInterface.js.map